#!/usr/bin/env python
class TimeoutError(Exception):

    """
    An exception raised when a test times out waiting for expected condition
    """


class ElementObjectNotSet(Exception):

    """
    An exception to be raised when an element_object has been not set or found
    """



from appiumdriver import AppiumDriver  # noqa
from element import Element  # noqa
from elements import Elements  # noqa
import exceptions  # noqa
from page import Page  # noqa
from testcase import AppiumTestCase  # noqa
from helpers import class_repr, class_string  # noqa



#!/usr/bin/env python
import logging

from appium.webdriver.common.touch_action import TouchAction
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import WebDriverException

from exceptions import ElementObjectNotSet

log = logging.getLogger(__name__)


class Element(object):

    def __init__(self, appium_driver, **kwargs):
        """
        Element object for wrapping webdriver element calls.
        Must pass a locator/locator_value in kwargs to find elements.
        :param: :appium_driver: webdriver object
        """
        self.appium_driver = appium_driver
        self.action = TouchAction(self.appium_driver)
        self.element_object = None

        if not kwargs:
            raise ValueError("Please specify a locator")
        if len(kwargs) > 1:
            raise ValueError("Please specify only one locator")
        locator_key, locator_value = next(iter(kwargs.items()))
        self.locator_value = locator_value
        self.locator_key = locator_key
        self.locator = (locator_key, locator_value)

    def find_element(self):
        """
        Function for finding element object for appium interaction.
        :return: webdriver element object
        """
        log.debug("Finding {}".format(self.locator))
        try:
            self.element_object = self.appium_driver.find_element(
                by=self.locator_key, value=self.locator_value)
        except NoSuchElementException as e:
            log.error(e)
            pass
        except WebDriverException:
            log.error(e)
            pass
        return self.element_object

    def is_displayed(self):
        """
        Check for if element is visible
        :return: Boolean
        """
        log.debug("Checking if {} is displayed".format(
            self.locator))
        self.find_element()
        return self.element_object.is_displayed() \
            if self.element_object else False

    def is_present(self):
        """
        Check for if element is present.
        :return: Boolean
        """
        log.debug("Checking if {} is present".format(self.locator))
        self.find_element()
        return self.element_object.is_present() \
            if self.element_object else False

    def is_enabled(self):
        """
        Check for if element is enabled
        :return: Boolean
        """
        log.debug("Checking if {} is enabled".format(self.locator))
        self.find_element()
        return self.element_object.is_enabled() \
            if self.element_object else False

    def tap(self, x=0, y=0, count=1):
        """
        Tap element object if element has already been found
        Example:
        self.element.find_element().tap()
        :param: :x: int offset x for tap
        :param: :y: int offset y for tap
        :param: :count: int number of times to tap
        """
        log.debug("Tapping {} with {},{} {} times".format(
            self.locator, x, y, count))
        if self.element_object:
            return self.action.tap(
                self.element_object, x=x, y=y, count=count).perform()

        raise ElementObjectNotSet

    def set_value(self, text):
        """
        Set value of element to provided string if element
        has been found.
        Example:
        self.element.find_element().set_value("foo")
        :param: :text: string to set element
        """
        log.debug("Setting value of {} to {}".format(
            self.locator, text))
        if self.element_object:
            return self.element_object.set_value(text)

        raise ElementObjectNotSet

    def send_keys(self, keys):
        """
        Send keys to element if element has been found.
        Example:
        self.element.find_element().send_keys("foo")
        :param: :keys: string of keys to send.
        """
        log.debug("Sending {} keys to {}".format(
            keys, self.locator))
        if self.element_object:
            return self.element_object.send_keys(keys)

        raise ElementObjectNotSet

    def text(self):
        """
        Get text of an element object.
        Example:
        self.element.find_element().text()
        """
        log.debug("Getting text of {}".format(self.locator))
        if self.element_object:
            return self.element_object.text

        raise ElementObjectNotSet



#!/usr/bin/env python
import logging

from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import WebDriverException

log = logging.getLogger(__name__)


class Elements(object):

    def __init__(self, appium_driver, **kwargs):
        """
        Element object for wrapping webdriver element calls.
        Must pass a locator/locator_value in kwargs to find elements.
        :param: :appium_driver: webdriver object
        """
        self.appium_driver = appium_driver
        self.element_objects = []

        if not kwargs:
            raise ValueError("Please specify a locator")
        if len(kwargs) > 1:
            raise ValueError("Please specify only one locator")
        locator_key, locator_value = next(iter(kwargs.items()))
        self.locator_value = locator_value
        self.locator_key = locator_key
        self.locator = (locator_key, locator_value)

    def find_elements(self):
        """
        Function for finding element objects for appium interaction.
        :return: webdriver element object
        """
        log.debug("Finding {}".format(self.locator))
        try:
            self.element_objects = self.appium_driver.find_elements(
                by=self.locator_key, value=self.locator_value)
        except NoSuchElementException as e:
            log.error(e)
            pass
        except WebDriverException:
            log.error(e)
            pass
        return self.element_objects



#!/usr/bin/env python
from abc import abstractmethod
import logging
import re

from element import Element
from elements import Elements


log = logging.getLogger(__name__)


class Page(object):
    """
    Page object implementation for an application page. Provides methods for
    element/page interactions.
    :param: :appium_driver: webdriver object
    """

    def __init__(self, appium_driver):
        self.appium_driver = appium_driver
        self.page_source = None

    @abstractmethod
    def verify(self):
        """
        Assert that we have arrived on the expected page.
        """
        return

    def create_element(self, **kwargs):
        """
        Create a page element object given a locator_key/value
        :param: :locator_key: String locator_key
        :param: :locator_value: String locator_value
        :return: Element object
        """
        log.debug("Creating element with {}".format(kwargs))
        element = Element(self.appium_driver, **kwargs)
        return element

    def create_elements(self, **kwargs):
        """
        Create a page element object given a locator_key/value
        :param: :locator_key: String locator_key
        :param: :locator_value: String locator_value
        :return: Element object
        """
        log.debug("Creating elements with {}".format(kwargs))
        elements = Elements(self.appium_driver, **kwargs)
        return elements

    def page_contains(self, value):
        """
        Search page_source string xml for a given substring value
        :param: :value: string value to find in page source
        :return: Boolean
        """
        log.debug("Check if page contains {}".format(value))
        does_contain_text = False
        text_regex = re.compile(re.escape(value))
        contains_text = text_regex.search(self.appium_driver.page_source())
        if contains_text:
            does_contain_text = True
        return does_contain_text



#!/usr/bin/env python
import logging
import os
import signal
from subprocess import Popen, PIPE
import sys
import time
import unittest

from appiumdriver import AppiumDriver
from exceptions import TimeoutError


log = logging.getLogger(__name__)

# DEFAULT CONFIGS
DEFAULT_TIMEOUT = 60
DEFAULT_WAIT = 1
DEFAULT_PLATFORM = "Android"
DEFAULT_PLATFORM_VERSION = "6.0"
DEFAULT_DEVICE_NAME = "Android Emulator"
TEST_ARTIFACTS_DIR = "{}/test-artifacts".format(os.getcwd())
# LOGGER CONFIGS
LOG_FORMAT = "[%(asctime)s] %(levelname)s " + \
    " [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
DATE_FORMAT = "%H:%M:%S"
LEVEL = "DEBUG"


class AppiumTestCase(unittest.TestCase):
    """
    Demo appium test case for Testworks Conf workshop.
    Must create a new test case in the ../tests folder.
    """

    def __init__(self, *args, **kwargs):
        super(AppiumTestCase, self).__init__(*args, **kwargs)
        self.appium_driver = None
        self.appium_proc = None

    def setUp(self, **kwargs):
        log.debug("Starting {}".format(self.__name__))
        test_start_time = self.get_current_time()
        # Make test run directory
        self.test_output_dir = os.path.join(TEST_ARTIFACTS_DIR,
                                            "{}_{}".format(
                                                self.__name__,
                                                test_start_time))
        if not os.path.exists(self.test_output_dir):
            os.makedirs(self.test_output_dir)
        # Set up test logger
        if log.root.handlers:
            log.root.handlers = []
        logging.basicConfig(
            filename="{}/test.log".format(self.test_output_dir),
            level=LEVEL, format=LOG_FORMAT, datefmt=DATE_FORMAT)
        # Stop/Start Appium Server
        self.stop_appium_server()
        self.start_appium_server()
        # Parse Capabilities
        self.app = os.getenv("APP")
        if not self.app:
            raise Exception("No test application set with export APP=")
        desired_capabilities = {
            "noSign": True,
            "app": self.app,
            "platformName": DEFAULT_PLATFORM,
            "platformVersion": DEFAULT_PLATFORM_VERSION,
            "deviceName": DEFAULT_DEVICE_NAME
        }

        # Connect to appium driver
        self.connect_to_appium(desired_capabilities)

    def tearDown(self, **kwargs):
        log.debug("Tearing down {}".format(self.__name__))
        sys_exc_info = sys.exc_info()
        test_passed = sys_exc_info == (None, None, None)

        # Check if tests passed and take final screenshot/page source if failed
        if not test_passed and self.appium_driver:
            log.error(
                "Failed! {}".format(sys_exc_info[1]))
            screenshot_filename = os.path.join(self.test_output_dir,
                                               "failure_screenshot.jpeg")
            self.appium_driver.save_screenshot(screenshot_filename)
            page_tree_file_name = os.path.join(self.test_output_dir,
                                               "page_tree.xml")
            with open(page_tree_file_name, 'w') as page_tree_file:
                element_tree = self.appium_driver.page_source()
                for line in element_tree:
                    page_tree_file.write(element_tree)

        # Quit appium driver
        if self.appium_driver:
            self.appium_driver.quit()

        # Stop appium server
        self.stop_appium_server()
        log.debug("{0} END {0}".format("=" * 25))

    def create_page(self, page_object):
        """
        Method for taking a page object and adding the appium_driver for the
        current test case to it for ease of instantiation.
        :param: :page_object: Page object to create
        :param: :validate: Boolean to validate page as part of creation
        """
        created_page = page_object(self.appium_driver)
        created_page.testcase = self
        return created_page

    def connect_to_appium(self, capabilities):
        """
        Open tests connection to appium driver on local host.
        :param: :capabilities: Dict of test capabilities.
        """
        webdriver_url = "http://localhost:4723/wd/hub"
        self.appium_driver = AppiumDriver(
            webdriver_url=webdriver_url,
            capabilities=capabilities)

    def start_appium_server(self):
        """
        Start appium server on localhost.
        """
        log.debug("Starting Appium Server")
        cmd = "appium --log {}/appium.log".format(self.test_output_dir)
        self.appium_proc = Popen(
            cmd, stdout=PIPE, shell=True, preexec_fn=os.setsid)
        # Need a hard sleep to wait for appium to start
        time.sleep(5)

    def stop_appium_server(self):
        """
        Stop appium server on localhost.
        """
        if self.appium_proc:
            log.debug("Stopping Appium: {}".format(self.appium_proc.pid))
            os.killpg(os.getpgid(self.appium_proc.pid), signal.SIGTERM)
        else:
            p = Popen(['ps', '-A'], stdout=PIPE)
            out, err = p.communicate()
            for process in out.splitlines():
                if 'appium' in process:
                    pid = int(process.split(None, 1)[0])
                    log.debug("Stopping Appium: {}".format(pid))
                    os.kill(pid, signal.SIGKILL)

    def get_current_time(self):
        """
        Return current time stamp.
        """
        return int(time.time())

    def wait_until(
            self,
            test_method,
            result=True,
            timeout_seconds=DEFAULT_TIMEOUT,
            seconds_between=DEFAULT_WAIT,
            *args,
            **kwargs):
        """
        Test method for asserting/retrying for a function to return expected
        condition.
        Examples:
        Wait till element is visible
        self.wait_until(self.element.is_visible())
        Wait till element is not visible
        self.wait_until(self.element.is_visible(), result=False)
        """
        log.debug("Waiting for {} to return {} for {} seconds".format(
            test_method.__name__, result, timeout_seconds))
        start = self.get_current_time()

        test_method_result = None
        while True:
            test_method_result = test_method(*args, **kwargs)
            if result is True and test_method_result:
                return True
            elif result == test_method_result:
                return True
            else:
                log.error("Method {} received response: {}\n"
                          "expected: {}.".format(
                              test_method.__name__,
                              test_method_result,
                              result))

            if (self.get_current_time() - start) > timeout_seconds:
                break

            time.sleep(seconds_between)

        raise TimeoutError("Failed waiting for {} to return {};\n"
                           "final value was {}".format(
                               test_method.__name__,
                               result, test_method_result))



#!/usr/bin/env python
import logging
import xml.dom.minidom

from appium import webdriver

log = logging.getLogger(__name__)


class AppiumDriver(webdriver.Remote):
    """
    AppiumDriver class
    Extension of webdriver class
    :param: :webdriver_url: String url for webdriver connection
    :param: :capabilities: Dict of desired capabilities of test
    """

    def __init__(self, webdriver_url=None, capabilities=None):
        self.webdriver_url = webdriver_url
        self.capabilities = capabilities
        log.debug("Connecting to Appium with: {}".format(capabilities))
        super(AppiumDriver, self).__init__(
            self.webdriver_url, self.capabilities)

    def page_source(self):
        """
        Pretty print of page_souce xml object
        :return: formatted xml string
        """
        log.debug("Collecting page source")
        element_source = super(AppiumDriver, self).page_source
        element_utf = element_source.encode('ascii', 'ignore').strip()
        xml_object = xml.dom.minidom.parseString(element_utf)
        return xml_object.toprettyxml('   ')



from setuptools import setup

__version__ = '0.1.0'
__url__ = 'https://github.com/apallin/testworks-appium/tarball/{}'.format(
    __version__)

setup(
    name='testworksappium',
    version=__version__,
    description="""
                A repository for Testworks Conf's Mobile Automation workshop
                presented by Adam Pallin
                """,
    author='Adam Pallin',
    author_email='adamrpallin@gmail.com',
    url='https://github.com/apallin/testworks-appium',
    packages=['testworksappium'],
    keywords=['appium', 'testworks'],
    download_url=__url__,
    install_requires=[
        'Appium-Python-Client >= 0.22, < 1.0',
    ],
)
